#include <Servo.h>
#include <Servo.h>
#include <NewPing.h>
#include <AFMotor.h>

#define TRIG_PIN_OBS A0
#define ECHO_PIN_OBS A3
#define TRIG_PIN_S A2
#define ECHO_PIN_S A5
#define MAX_DIST 400
#define MAX_SPEED 255

NewPing sonarObs(TRIG_PIN_OBS, ECHO_PIN_OBS, MAX_DIST);
NewPing sonarS(TRIG_PIN_S, ECHO_PIN_S, MAX_DIST);

AF_DCMotor motorRight(2,  MOTOR12_8KHZ);
AF_DCMotor motorLeft(3,  MOTOR12_8KHZ);
Servo servo;

char dataIn = 'S';
int speed = 255;
int dist = 100;
int distS = 0;

void turnOffMotors() {
  motorRight.run(RELEASE);
  motorLeft.run(RELEASE);
}

void moveForward() {
    motorRight.setSpeed(speed);
    motorLeft.setSpeed(speed);
    motorRight.run(FORWARD);      
    motorLeft.run(FORWARD);
}

void moveBackward() {
    motorRight.setSpeed(speed);
    motorLeft.setSpeed(speed);
    motorRight.run(BACKWARD);      
    motorLeft.run(BACKWARD);
}

void turnLeft() {
  motorRight.setSpeed(speed);
  motorLeft.setSpeed(speed);
  motorRight.run(FORWARD);      
  motorLeft.run(BACKWARD);
}

void turnRight() {
  motorRight.setSpeed(speed);
  motorLeft.setSpeed(speed);
  motorRight.run(BACKWARD);      
  motorLeft.run(FORWARD);
}

void moveLeft() {
  motorRight.setSpeed(speed);
  motorLeft.setSpeed(speed);
  motorRight.run(FORWARD);      
  motorLeft.run(BACKWARD);
  delay(500);
  motorRight.setSpeed(speed);
  motorLeft.setSpeed(speed);
  motorRight.run(FORWARD);      
  motorLeft.run(FORWARD);
}

void moveRight() {
  motorRight.setSpeed(speed);
  motorLeft.setSpeed(speed);
  motorRight.run(BACKWARD);      
  motorLeft.run(FORWARD);
  delay(500);
  motorRight.setSpeed(speed);
  motorLeft.setSpeed(speed);
  motorRight.run(FORWARD);      
  motorLeft.run(FORWARD);
}

void moveRightForward() {
  motorRight.setSpeed(speed / 2);
  motorLeft.setSpeed(speed);
  motorRight.run(FORWARD);      
  motorLeft.run(FORWARD);
}

void moveRightBackward() {
  motorRight.setSpeed(speed / 2);
  motorLeft.setSpeed(speed);
  motorRight.run(BACKWARD);      
  motorLeft.run(BACKWARD);
}

void moveLeftForward() {
  motorRight.setSpeed(speed);
  motorLeft.setSpeed(speed / 2);
  motorRight.run(FORWARD);      
  motorLeft.run(FORWARD);
}

void moveLeftBackward() {
  motorRight.setSpeed(speed);
  motorLeft.setSpeed(speed / 2);
  motorRight.run(BACKWARD);      
  motorLeft.run(BACKWARD);
}

void initServo() {
  servo.attach(10);
  servo.write(90);
}

char getBluetoothInput() {
  if (Serial.available() > 0) {
    dataIn = Serial.read();
  }
  return dataIn;
}

int getRightDist() {
  servo.write(0);
  delay(500);
  delay(70);
  int dist = sonarObs.ping_cm();
  delay(100);
  servo.write(90);
  return dist;
}

int getLeftDist() {
  servo.write(180);
  delay(500);
  delay(70);
  int dist = sonarObs.ping_cm();
  delay(100);
  servo.write(90);
  return dist;
}

void avoid() {
  distS = sonarS.ping_cm();
      delay(40);
      if (distS >= 7) {
        turnOffMotors();
        delay(300);
        moveBackward();
        delay(50);
        turnLeft();
        delay(100);
      } else {
        int rDist = 0;
        int lDist = 0;
        dist = sonarObs.ping_cm();
        delay(40);
        if (dist <= 25) {
          // obstacle is too close and needs to be avoided
          Serial.println("obstacle is too close and needs to be avoided");

          turnOffMotors();
          delay(100);

          moveBackward();
          delay(300);

          turnOffMotors();
          delay(200);

          rDist = getRightDist();
          delay(200);
          lDist = getLeftDist();
          delay(200);

          if (rDist >= lDist) {
            moveRight();
            turnOffMotors();
          } else {
            moveLeft();
            turnOffMotors();
          }
        } else {
          Serial.println("No obstacles on my way");
          delay(15);
          moveForward();
        }
      }
      dist = sonarObs.ping_cm();
}

void setup() {
  // put your setup code here, to run once:
  Serial.begin(9600);
  initServo();
  turnOffMotors();
}

void loop() {
  // put your main code here, to run repeatedly:
  char command = getBluetoothInput();
  Serial.println(command);

  switch (command) {
    case 'F':
      moveForward();
      command = getBluetoothInput();
      break;

    case 'B':;
      moveBackward();
      command = getBluetoothInput();
      break;

    case 'L':
      turnLeft();
      command = getBluetoothInput();
      break;

    case 'R':
      turnRight();
      command = getBluetoothInput();
      break;

    case 'I':
      moveRightForward();
      command = getBluetoothInput();
      break;

    case 'J':
      moveRightBackward;
      command = getBluetoothInput();
      break;


    case 'G':
      moveLeftForward();
      command = getBluetoothInput();
      break;

    case 'H':
      moveLeftBackward();
      command = getBluetoothInput();
      break;

    case 'S':
      turnOffMotors();
      command = getBluetoothInput();
      break;

    case 'W':
      servo.write(90);
      command = getBluetoothInput();
      break;

    case '1':
      servo.write(45);
      command = getBluetoothInput();
      break;

    case '2':
      servo.write(0);
      command = getBluetoothInput();
      break;
    
    case '3':
      servo.write(135);
      command = getBluetoothInput();
      break;
    
    case '4':
      servo.write(180);
      command = getBluetoothInput();
      break;

    case '5':
      speed = 150;
      command = getBluetoothInput();
      break;
    
      case '6':
      speed = 175;
      command = getBluetoothInput();
      break;

      case '7':
      speed = 200;
      command = getBluetoothInput();
      break;

      case '8':
      speed = 225;
      command = getBluetoothInput();
      break;

      case '9':
      speed = 250;
      command = getBluetoothInput();
      break;

      case 'V':
        avoid();        
        command = getBluetoothInput();
        break;
  }
}
